//
//  CameraDetectManager.swift
//  DemoSwift_Example
//
//  Created by siming tang on 2022/3/11.
//  Copyright © 2022 CocoaPods. All rights reserved.
//

import Foundation
import AVFoundation
import UIKit

protocol CameraDetectDelegate: AnyObject {
    //    相机图片
    func didOutputImage(_ image: UIImage)
    //    检测到光亮
    func didDetectLight(_ value: Double)
    //    检测到人脸
    func didDetectFaceImage(_ images: [(OriginSize, UIImage)])
    // 检测失败
    func didFailed(_ msg: String)
}

// 光检测
class CameraDetectManager: NSObject {
   
    weak var delegate: CameraDetectDelegate?
    /// 检测间隔
    var detectInterval: TimeInterval = 1 / 24
    
    func startDetect() {
        self.__queryAuthorized()
    }
    
    func stopDetect() {
        self.currentSession?.stopRunning()
    }
    
    private var currentSession: AVCaptureSession?
    /// 最新检测时间
    private var lastDetectDate: Date?
}

extension CameraDetectManager {
    
    private func __queryAuthorized() {
        let status = AVCaptureDevice.authorizationStatus(for: .video)
        switch status {
        case .notDetermined:
            DLog("相机权限未知")
            AVCaptureDevice.requestAccess(for: .video) { isAllow in
                if isAllow {
                    self.__start()
                }
                else {
                    
                }
            }
        case .authorized:
            DLog("相机权限允许")
            self.__start()
        case .denied:
            DLog("相机权限拒绝")
        case .restricted:
            DLog("相机权限受限")
        default:
            DLog("相机权限未知")
        }
    }
    
    private func __start() {
        // 1.获取硬件设备
        guard let device = AVCaptureDevice.default(.builtInWideAngleCamera, for: .video, position: .front) else {
            DLog("获取设备失败")
            self.delegate?.didFailed("获取摄像头失败")
            return
        }
        
        // 2.创建输入流
        guard let input = try? AVCaptureDeviceInput.init(device: device) else {
            DLog("创建输入流失败")
            self.delegate?.didFailed("获取摄像头输入流失败")
            return
        }
        
        // 3.创建设备输出流
        let output = AVCaptureVideoDataOutput()
        // 设置为高质量采集率
        output.setSampleBufferDelegate(self, queue: DispatchQueue.global())
        
        let session = AVCaptureSession()
        session.sessionPreset = .photo
        self.currentSession = session
        
        // 添加会话输入和输出
        guard session.canAddInput(input) else {
            DLog("会话添加输入流失败")
            return
        }
        guard session.canAddOutput(output) else {
            DLog("会话添加输出流失败")
            return
        }
        
        session.addInput(input)
        session.addOutput(output)
        
        // 9.启动会话
        session.startRunning()
        
    }
    
    private func __getLightValue(sampleBuffer: CMSampleBuffer) {
        if let metadataDict: Dictionary = CMCopyDictionaryOfAttachments(allocator: nil, target: sampleBuffer, attachmentMode: kCMAttachmentMode_ShouldPropagate) as? Dictionary<String, Any> {
            if let exifMetadataDict: Dictionary = metadataDict[kCGImagePropertyExifDictionary as String] as? Dictionary<String, Any> {
                if let lightValue: Double = exifMetadataDict[kCGImagePropertyExifBrightnessValue as String] as? Double {
//                    DLog("光亮度：\(lightValue)")
                    dispatchMainAsyncSafe {
                        self.delegate?.didDetectLight(lightValue)
                    }
                }
            }
        }
    }
    
    private func __getImage(sampleBuffer: CMSampleBuffer, connection: AVCaptureConnection) {
        if let image = UIImage.imageFromSampleBuffer(sampleBuffer: sampleBuffer, videoOrientation: connection.videoOrientation) {
            dispatchMainAsyncSafe {
                self.delegate?.didOutputImage(image)
            }
//            DLog("图片整大小width:\(image.size.width), height:\(image.size.height)")
            self.__convertToFaceImages(image: image)
        }
    }
    
    private func __convertToFaceImages(image: UIImage) {
        // 获取裁剪人脸
        image.faceeDetect { result in
            switch result {
            case let .success(faceImages):
                DLog("检测到人脸\(faceImages.count)张")
                dispatchMainAsyncSafe {
                    self.delegate?.didDetectFaceImage(faceImages)
                }
            case .notFound:
                DLog("未检测到")
            case let .failure(error):
                DLog("检测失败")
                dispatchMainAsyncSafe {
                    self.delegate?.didFailed("检测人脸错误：\(error?.localizedDescription ?? "")")
                }
            }
        }
    }
}

extension CameraDetectManager: AVCaptureVideoDataOutputSampleBufferDelegate {
    func captureOutput(_ output: AVCaptureOutput, didOutput sampleBuffer: CMSampleBuffer, from connection: AVCaptureConnection) {
        //        connection.videoOrientation
        if let lastDate = self.lastDetectDate, Date().timeIntervalSince1970 - lastDate.timeIntervalSince1970 < self.detectInterval {
            return
        }
        self.lastDetectDate = Date()
        self.__getLightValue(sampleBuffer: sampleBuffer)
        self.__getImage(sampleBuffer: sampleBuffer, connection: connection)
    }
}
