//
//  UIImage+FaceDetect.swift
//  DemoSwift_Example
//
//  Created by siming tang on 2022/3/14.
//  Copyright © 2022 CocoaPods. All rights reserved.
//

import Foundation
import Vision
import UIKit

typealias OriginSize = (center: CGPoint, faceSize: CGSize, imgSize: CGSize)

enum FaceDetectResult {
    case success([(OriginSize, UIImage)])
    case notFound
    case failure(Error?)
}

/// 人脸检测
@available(iOS 11.0, *)
extension UIImage {
    
    /// 人脸检测
    /// - Parameter completion: 返回检测结果类型，成功返回人脸裁剪图片
    func faceeDetect(_ completion: @escaping ((FaceDetectResult) -> Void)) {
        
        guard let tmpCgImage = self.cgImage else {
            completion(.notFound)
            return
        }
        
        let req = VNDetectFaceRectanglesRequest { request, error in
            guard error == nil else {
                completion(.failure(error))
                return
            }
            
            let faceImages = request.results?.map({ result -> (OriginSize, UIImage)? in
                guard let face = result as? VNFaceObservation else { return nil }
                guard let faceImage = self.__cropImage(object: face) else {return nil}
                let originSize = self.__getOriginSize(object: face)
                return (originSize, UIImage.init(cgImage: faceImage))
            }).compactMap { $0 }
                        
            guard let result = faceImages, result.count > 0 else {
                completion(.notFound)
                return
            }
            
            completion(.success(result))
        }
        
        do {
            try VNImageRequestHandler(cgImage: tmpCgImage, options: [:]).perform([req])
        } catch let error {
            completion(.failure(error))
        }
    }
    
    
    //    裁剪图片
    private func __cropImage(object: VNDetectedObjectObservation) -> CGImage? {
        let width = object.boundingBox.width * CGFloat(self.size.width)
        let height = object.boundingBox.height * CGFloat(self.size.height)
        let x = object.boundingBox.origin.x * CGFloat(self.size.width)
        let y = (1 - object.boundingBox.origin.y) * CGFloat(self.size.height) - height
//        DLog("人脸:x:\(x),y:\(y),w:\(width),h:\(height)")
        let croppingRect = CGRect(x: x, y: y, width: width, height: height)
        let image = self.cgImage?.cropping(to: croppingRect)
        return image
    }
    
    
    /// 获取中心点、尺寸等参数
    private func __getOriginSize(object: VNDetectedObjectObservation) -> OriginSize {
        let faceRect = object.boundingBox
        let cX = faceRect.midX * self.size.width
        let cY = faceRect.midY * self.size.height
        let w = faceRect.width * self.size.width
        let h = faceRect.height * self.size.height
        return (center: CGPoint.init(x: cX, y: cY), faceSize: CGSize.init(width: w, height: h), self.size)
    }
}
